/*
 *
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.ars.ars.
 * 
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.

 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436 
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.backup.plugins;

import de.bsvrz.ars.ars.backup.BackupException;
import de.bsvrz.sys.funclib.kappich.annotations.NotNull;

import java.io.File;
import java.util.Properties;

/**
 * Diese Klasse ist für die Sicherung von Containern auf dem Filesystem verantwortlich.
 *
 * @author beck et al. projects GmbH
 * @author Christian Wied
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class FileSystemBackup extends AbstractFileSystemBackup {


	/** @see de.bsvrz.ars.ars.backup.BackupModul#initialize(java.util.Properties) */
	@Override
	public void initialize(Properties props) throws Exception {

		String zipSize = props.getProperty(PROP_BACKUP_CONTAINER_PER_ZIP);
		try {
			if(zipSize != null) {
				MAX_CONTAINERS_PER_DIR = Integer.parseInt(zipSize);
			}
		}
		catch(NumberFormatException e) {
		} // do nothing, default 100 is ok.

		String mediumSize = props.getProperty(PROP_BACKUP_MEDIUMSIZE);
		try {
			if(mediumSize != null) {
				MAX_MEDIUM_SIZE_KB = Long.parseLong(mediumSize);
			}
		}
		catch(NumberFormatException e) {
		} // do nothing, take default.

		String backupDirName = props.getProperty(PROP_BACKUP_DIR);

		if(backupDirName == null) {
			throw new Exception("Benötigter Parameter '" + PROP_BACKUP_DIR + "' nicht angegeben.");
		}
		File backupDir = new File(backupDirName);
		if(!backupDir.exists()) {
			backupDir.mkdir();
		}
		backupBasePath = backupDir.getAbsolutePath();
	}

	/** @see de.bsvrz.ars.ars.backup.BackupModul#hasMediumCapacity(int,long,int) */
	@Override
	public boolean hasMediumCapacity(int mediumID, long containerSize, int mediumIndexSize) throws BackupException {
		return MAX_MEDIUM_SIZE_KB * 1024L > currentSpaceOccupiedByZipfiles + currentSpaceOccupiedByContainerFiles + containerSize + mediumIndexSize;
	}

	@Override
	@NotNull
	public String getBackupPath(final int mediumID) {
		return backupBasePath + File.separator + "Medium_" + mediumID;
	}

	public String toString() {
		return "FileSystem (" + backupBasePath + ")";
	}
}
