/*
 *
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.ars.ars.
 * 
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.

 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436 
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.mgmt.commands;

import de.bsvrz.ars.ars.mgmt.ArchiveManager;
import de.bsvrz.sys.funclib.losb.util.cmdinterface.Command;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * Klasse zur zyklischen Abfrage der Archiv-Warteschlangen
 *
 * @author beck et al. projects GmbH
 * @author Alexander Schmidt
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class ArchiveMonitor extends Thread {

	private static String BAR = "##################################################";

	private static String BLANK = "                                                  ";

	public static final DateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy  HH:mm:ss,SSS");

	private int uInt;

	private ArchiveManager aMgr;

	private Command cmd;

	private boolean terminated = false;

	public ArchiveMonitor(ArchiveManager am, int updateInterval, Command command) {
		aMgr = am;
		uInt = updateInterval;
		cmd = command;
	}

	public void terminate() {
		terminated = true;
		interrupt();
	}


	public void run() {
		float[][] res = null;
		float[][] old = null;
		while(!terminated) {
			try {
				old = res;
				res = aMgr.getInQueuesMgr().getArchiveQueuesOnlineStatus();
				String[] status = new String[res.length];

				if(areDifferent(res, old)) {
					float maxQSize = 0, summe = 0;
					for(int i = 0; i < res.length; i++) {
						maxQSize = Math.max(maxQSize, res[i][0]);
						summe += res[i][0];
					}
					for(int i = 0; i < res.length; i++) {
						String cross = BAR.substring(0, (int)((res[i][0] / maxQSize) * BAR.length()));
						status[i] = String.format("Queue #%4s: %7d ", i, (int)res[i][0]) + "[" + cross + BLANK.substring(0, BLANK.length() - cross.length())
						            + "] " + String.format("%3d%%", (int)((res[i][0] / summe) * 100));
					}
					try {
						cmd.printlnPlain("               ------------- " + dateFormat.format(new Date()) + " -------------");
						for(int i = 0; i < status.length; i++) {
							cmd.printlnPlain(status[i]);
						}
						int fraction = (int)((summe / aMgr.getInQueuesMgr().getTotalOnlineQueuesCapa()) * BAR.length());
						cmd.printlnPlain(
								String.format("SUMME: %12d ", (int)summe) + "[" + BAR.substring(0, fraction) + BLANK.substring(0, BLANK.length() - fraction)
								+ "] " + String.format("%3d%%", (int)((summe / aMgr.getInQueuesMgr().getTotalOnlineQueuesCapa()) * 100))
						);
					}
					catch(Exception e) {
						break;
					}
				}
				sleep(uInt);
			}
			catch(InterruptedException e) {
				break;
			}
		}
	}


	private boolean areDifferent(float[][] a, float[][] b) {
		// Basis check
		if(a == null && b != null || a != null && b == null) return true;
		if(a == null && b == null) return false;
		if(a.length != b.length) return true;

		// Vergleich der Werte
		for(int i = 0; i < a.length; i++) {
			if(a[i][0] != b[i][0]) return true;
		}
		return false;
	}
}
