/*
 *
 * Copyright 2017 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.ars.ars.
 * 
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.

 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436 
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.persistence.index.result;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Ergebnis einer Indexanfrage. Enthält eine Liste von Container-IDs jeweils mit Min/Max-Werten. Bei Anfragen an den monotonen Index sind Min/Max-Werte für
 * Datenindex, Archivzeit und Datenzeit enthalten, bei Anfragen an den Datenzeitindex nur Min/Max-Werte für Datenzeit und Datenindex (die Archivzeit wurde nicht
 * benoetigt und deshalb auch nicht im Datenzeitindex gespeichert). Im letzteren Fall sind die Min/Max-Werte der Archivzeit auf {@link #NO_VALUE} gesetzt.
 *
 * @author beck et al. projects GmbH
 * @author Alexander Schmidt
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class IndexResult<T extends ContainerIndexResult> {

	public static final int NO_VALUE = -1;

	public static final IndexResult<?> EMPTY;

	static {
		EMPTY = new IndexResult<>();
		EMPTY._entries = Collections.emptyList();
	}

	private List<T> _entries = new ArrayList<>();

	private IndexResult() {
	}

	public static IndexResult<ArchiveTimeIndexEntry> newArchiveTimeIndexResult() {
		return new IndexResult<>();
	}

	public static IndexResult<DataTimeIndexEntry> newDataTimeIndexResult() {
		return new IndexResult<>();
	}

	public static IndexResult<DataIndexIndexEntry> newDataIndexIndexResult() {
		return new IndexResult<>();
	}

	/**
	 * Fuegt einen Container mit den angegebenen Min/Max-Werten der Ergebnisliste hinzu.
	 *
	 * @param entry
	 */
	public void add(final T entry) {
		_entries.add(entry);
	}

	@Deprecated
	public void updateDIMax(int pos, long diMax) {
		assert pos < _entries.size() : "Ungültige Position";
		ContainerIndexResult tmp = get(pos);
		_entries.set(pos, (T) tmp.withDiMax(diMax));
	}

	/**
	 * Fuegt einen Container mit den angegebenen Min/Max-Werten in der Ergebnisliste an angegebener Stelle hinzu
	 *
	 * @param pos
	 * @param entry
	 */
	public void insert(int pos, final T entry) {
		_entries.add(pos, entry);
	}


	/** @return Anzahl der enthaltenen Containerinformationen */
	public int size() {
		return _entries.size();
	}

	/** @return Wahr, wenn keine Container in der Ergebnisliste enthalten sind, falsch sonst */
	public boolean isEmpty() {
		return size() == 0;
	}

	public T get(final int index) {
		return _entries.get(index);
	}

	public String toString() {
		return _entries.toString();
	}

}
