/*
 *
 * Copyright 2017 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.ars.ars.
 * 
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.

 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436 
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.persistence.iter;

import de.bsvrz.ars.ars.mgmt.ArchiveManager;
import de.bsvrz.ars.ars.mgmt.tasks.Task;
import de.bsvrz.ars.ars.persistence.ContainerDataResult;
import de.bsvrz.ars.ars.persistence.IdDataIdentification;
import de.bsvrz.ars.ars.persistence.PersistenceException;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveOrder;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * TBD Dokumentation
 *
 * @author Kappich Systemberatung
 */
public class BucketContainerIterator implements DataIterator {

	private final ArchiveManager _archiveManager;
	private final List<List<QueryDataSequence.IndexedContainer>> _containerFiles;
	private final IdDataIdentification _dataIdentification;
	private final ArchiveDataKind _adk;
	private final Task _task;
	private DataIterator _currentDelegate;
	private int _index = 0;
	private ArchiveOrder _order;
	private DeletedTreatment _deletedTreatment;

	public BucketContainerIterator(final ArchiveManager archiveManager, final List<List<QueryDataSequence.IndexedContainer>> containerFiles, final IdDataIdentification dataIdentification, final ArchiveDataKind adk, final DeletedTreatment deletedTreatment, final Task task, final ArchiveOrder order) throws PersistenceException {
		_archiveManager = archiveManager;
		_containerFiles = containerFiles;
		_dataIdentification = dataIdentification;
		_adk = adk;
		_deletedTreatment = deletedTreatment;
		_task = task;
		_order = order;
		if(containerFiles.isEmpty()) throw new IllegalArgumentException("containerFiles");
		openBucket(_index);
	}


	private void openBucket(final int index) throws PersistenceException {
		List<QueryDataSequence.IndexedContainer> bucket = _containerFiles.get(index);
		if(bucket.size() == 1) {
			_currentDelegate = new SequentialContainerIterator(_archiveManager, bucket, _dataIdentification, _adk, _deletedTreatment, _task);
		}
		else {
			final List<DataIterator> iterators = new ArrayList<>();
			for(QueryDataSequence.IndexedContainer indexedContainer : bucket) {
				iterators.add(new SequentialContainerIterator(_archiveManager, Collections.singletonList(indexedContainer), _dataIdentification, _adk, _deletedTreatment, _task));
			}
			_currentDelegate = new CombineAdkDataIterator(iterators, _order);
		}
	}

	private void closeBucket(final int index) throws PersistenceException {
		_currentDelegate.close();
	}

	@Override
	public void peek(final ContainerDataResult result) throws PersistenceException {
		_currentDelegate.peek(result);
	}

	@Override
	public long peekDataIndex() {
		return _currentDelegate.peekDataIndex();
	}

	@Override
	public long peekDataTime() {
		return _currentDelegate.peekDataTime();
	}

	@Override
	public long peekArchiveTime() {
		return _currentDelegate.peekArchiveTime();
	}

	@Override
	public void remove() throws PersistenceException {
		_currentDelegate.remove();
		if(_currentDelegate.isEmpty() && _index < _containerFiles.size() - 1) {
			closeBucket(_index);
			_index++;
			openBucket(_index);
		}
	}

	@Override
	public boolean isEmpty() {
		return _currentDelegate.isEmpty();
	}

	@Override
	public void close() throws PersistenceException {
		_currentDelegate.close();
	}

	@Override
	public long getCurContFileIdx() {
		return _currentDelegate.getCurContFileIdx();
	}
}
